@@ -26,10 +26,60 @@
  * EMail: roseg@apsis.ch
  */
 
-static char *rcs_id = "$Id: config.c,v 1.6 2003/11/30 22:56:26 roseg Rel $";
+static char *rcs_id = "$Id: config.c,v 1.9 2005/06/01 15:01:53 roseg Rel roseg $";
 
 /*
  * $Log: config.c,v $
+ * Revision 1.9  2005/06/01 15:01:53  roseg
+ * Enhancements:
+ *   Added the VerifyList configuration flag (CA root certs + CRL)
+ *   CRL checking code
+ *   RewriteRedirect 2 - ignores port value for host matching
+ *   Added -c flag (check-only mode)
+ *   Added -v flag (verbose mode)
+ *   Added -p flag for pid file name
+ *
+ * Bug fixes:
+ *   fixed a potential buffer overflow problem (in checking the Host header)
+ *   added call to SSL_library_init
+ *   added a check for MSIE before forcing SSL shutdown
+ *   X-SSL-Cipher header is added only if HTTPSHeaders is non-zero
+ *   added code for shorter linger on badly closed connections (IE work-around)
+ *   fixed the locking for session checking (mutex_lock/unlock)
+ *
+ * Revision 1.8  2004/11/04 13:37:07  roseg
+ * Changes:
+ * - added support for non-blocking connect(2)
+ * - added support for 414 - Request URI too long
+ * - added RedirectRewrite directive - to prevent redirect changes
+ * - added support for NoHTTPS11 value 2 (for MSIE clients only)
+ * - added support for HTTPSHeaders 3 (no verify)
+ *
+ * Problems fixed:
+ * - fixed bug if multiple listening ports/addresses
+ * - fixed memory leak in SSL
+ * - flush stdout (if used) after each log message
+ * - assumes only 304, 305 and 306 codes to have no content
+ * - fixed problem with delays in 302 without content
+ * - fixed problem with time-outs in HTTPS
+ *
+ * Enhancements:
+ * - improved threads detection code in autoconf
+ * - added supervisor process disable configuration flag
+ * - tweak for the Location rewriting code (only look at current GROUP)
+ * - improved print-out for client certificate information
+ *
+ * Revision 1.7  2004/03/24 06:59:59  roseg
+ * Fixed bug in X-SSL-CIPHER description
+ * Changed README to stx format for consistency
+ * Addedd X-SSL-certificate with full client certificate
+ * Improved the response times on HTTP/0.9 (content without Content-length)
+ * Improved response granularity on above - using unbuffered BIO now
+ * Fixed problem with IE/SSL (SSL_set_shutdown)
+ * Avoid error messages on premature EOF from client
+ * Fixed HeadRemove code so all headers are checked without exception
+ * Improved autoconf detection
+ *
  * Revision 1.6  2003/11/30 22:56:26  roseg
  * Callback for RSA ephemeral keys:
  *     - generated in a separate thread
@@ -149,6 +199,7 @@ static char *rcs_id = "$Id: config.c,v 1.6 2003/11/30 22:56:26 roseg Rel $";
  *
  */
 
+#define SYSLOG_NAMES    1
 #include    "pound.h"
 
 static char *
@@ -186,9 +237,10 @@ parse_file(char *fname)
     char                lin[MAXBUF], pat[MAXBUF];
     regex_t             Empty, Comment, ListenHTTP, ListenHTTPS, HTTPSHeaders, MaxRequest, HeadRemove,
                         SSL_CAlist, SSLEngine, SessionIP, SessionURL, SessionCOOKIE, SessionBASIC, NO11SSL,
-                        User, Group, RootJail, ExtendedHTTP, WebDAV, LogLevel, Alive, Server,
-                        Client, UrlGroup, HeadRequire, HeadDeny, BackEnd, BackEndHA, EndGroup,
-                        Err500, Err501, Err503, CheckURL, CS_SEGMENT, CS_PARM, CS_QID, CS_QVAL, CS_FRAG;
+                        SSL_Verifylist, User, Group, RootJail, ExtendedHTTP, WebDAV, LogFacility, LogLevel, Alive,
+                        Server, Client, UrlGroup, HeadRequire, HeadDeny, BackEnd, BackEndHA, EndGroup,
+                        Err500, Err501, Err503, Err414, CheckURL, CS_SEGMENT, CS_PARM, CS_QID, CS_QVAL, CS_FRAG,
+                        RewriteRedir, NoDaemon;
     regex_t             *req, *deny;
     regmatch_t          matches[5];
     struct sockaddr_in  addr, alive_addr;
@@ -201,8 +253,9 @@ parse_file(char *fname)
     || regcomp(&ListenHTTP, "^[ \t]*ListenHTTP[ \t]+([^,]+,[1-9][0-9]*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&ListenHTTPS, "^[ \t]*ListenHTTPS[ \t]+([^,]+,[1-9][0-9]*)[ \t]+([^ \t]+)[ \t]*([^ \t]*)[ \t]*$",
         REG_ICASE | REG_NEWLINE | REG_EXTENDED)
-    || regcomp(&HTTPSHeaders, "^[ \t]*HTTPSHeaders[ \t]+([012])[ \t]+\"([^\"]*)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
-    || regcomp(&SSL_CAlist, "^[ \t]*CAlist[ \t]+([^ \t]+)[ \t]+([0-9])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&HTTPSHeaders, "^[ \t]*HTTPSHeaders[ \t]+([0123])[ \t]+\"([^\"]*)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&SSL_CAlist, "^[ \t]*CAlist[ \t]+([^ \t]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&SSL_Verifylist, "^[ \t]*VerifyList[ \t]+([^ \t]+)[ \t]+([0-9])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
 #if HAVE_OPENSSL_ENGINE_H
     || regcomp(&SSLEngine, "^[ \t]*SSLEngine[ \t]+([^ \t]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
 #endif
@@ -218,7 +271,8 @@ parse_file(char *fname)
     || regcomp(&RootJail, "^[ \t]*RootJail[ \t]+([^ \t]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&ExtendedHTTP, "^[ \t]*ExtendedHTTP[ \t]+([01])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&WebDAV, "^[ \t]*WebDAV[ \t]+([01])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
-    || regcomp(&NO11SSL, "^[ \t]*NoHTTPS11[ \t]+([01])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&NO11SSL, "^[ \t]*NoHTTPS11[ \t]+([012])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&LogFacility, "^[ \t]*LogFacility[ \t]+([a-z0-9]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&LogLevel, "^[ \t]*LogLevel[ \t]+([01234])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&Alive, "^[ \t]*Alive[ \t]+([1-9][0-9]*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&Client, "^[ \t]*Client[ \t]+([1-9][0-9]*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
@@ -236,6 +290,7 @@ parse_file(char *fname)
     || regcomp(&Err500, "^[ \t]*Err500[ \t]+\"([^\"]+)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&Err501, "^[ \t]*Err501[ \t]+\"([^\"]+)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&Err503, "^[ \t]*Err503[ \t]+\"([^\"]+)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&Err414, "^[ \t]*Err414[ \t]+\"([^\"]+)\"[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&CheckURL, "^[ \t]*CheckURL[ \t]+([01])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&CS_SEGMENT, "^[ \t]*CSsegment[ \t]+([^ ]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&CS_PARM, "^[ \t]*CSparameter[ \t]+([^ ]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
@@ -243,6 +298,8 @@ parse_file(char *fname)
     || regcomp(&CS_QVAL, "^[ \t]*CSqval[ \t]+([^ ]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&CS_FRAG, "^[ \t]*CSfragment[ \t]+([^ ]+)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&MaxRequest, "^[ \t]*MaxRequest[ \t]+([1-9][0-9]*)[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&RewriteRedir, "^[ \t]*RewriteRedirect[ \t]+([0-2])[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
+    || regcomp(&NoDaemon, "^[ \t]*NoDaemon[ \t]*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     ) {
         logmsg(LOG_ERR, "bad config Regex - aborted");
         exit(1);
@@ -369,6 +426,12 @@ parse_file(char *fname)
                 logmsg(LOG_ERR, "CAlist config: out of memory - aborted");
                 exit(1);
             }
+        } else if(!regexec(&SSL_Verifylist, lin, 4, matches, 0)) {
+            lin[matches[1].rm_eo] = '\0';
+            if((ssl_Verifylst = strdup(lin + matches[1].rm_so)) == NULL) {
+                logmsg(LOG_ERR, "Verifylist config: out of memory - aborted");
+                exit(1);
+            }
             ssl_vdepth = atoi(lin + matches[2].rm_so);
         } else if(!regexec(&Err500, lin, 4, matches, 0)) {
             lin[matches[1].rm_eo] = '\0';
@@ -379,6 +442,9 @@ parse_file(char *fname)
         } else if(!regexec(&Err503, lin, 4, matches, 0)) {
             lin[matches[1].rm_eo] = '\0';
             e503 = file2str(lin + matches[1].rm_so);
+        } else if(!regexec(&Err414, lin, 4, matches, 0)) {
+            lin[matches[1].rm_eo] = '\0';
+            e414 = file2str(lin + matches[1].rm_so);
         } else if(!regexec(&User, lin, 4, matches, 0)) {
             lin[matches[1].rm_eo] = '\0';
             if((user = strdup(lin + matches[1].rm_so)) == NULL) {
@@ -460,6 +526,13 @@ parse_file(char *fname)
             allow_dav = atoi(lin + matches[1].rm_so);
         } else if(!regexec(&NO11SSL, lin, 4, matches, 0)) {
             no_https_11 = atoi(lin + matches[1].rm_so);
+        } else if(!regexec(&LogFacility, lin, 4, matches, 0)) {
+            lin[matches[1].rm_eo] = '\0';
+            for(k = 0; facilitynames[k].c_name; k++)
+                if(!strcmp(facilitynames[k].c_name, lin + matches[1].rm_so)) {
+                    log_facility = facilitynames[k].c_val;
+                    break;
+                }
         } else if(!regexec(&LogLevel, lin, 4, matches, 0)) {
             log_level = atoi(lin + matches[1].rm_so);
         } else if(!regexec(&Alive, lin, 4, matches, 0)) {
@@ -472,6 +545,10 @@ parse_file(char *fname)
             max_req = atol(lin + matches[1].rm_so);
         } else if(!regexec(&CheckURL, lin, 4, matches, 0)) {
             check_URL = atoi(lin + matches[1].rm_so);
+        } else if(!regexec(&RewriteRedir, lin, 4, matches, 0)) {
+            rewrite_redir = atoi(lin + matches[1].rm_so);
+        } else if(!regexec(&NoDaemon, lin, 4, matches, 0)) {
+            daemonize = 0;
         } else if(!regexec(&UrlGroup, lin, 4, matches, 0)) {
             if(in_group) {
                 logmsg(LOG_ERR, "UrlGroup in UrlGroup - aborted");
@@ -498,7 +575,7 @@ parse_file(char *fname)
             memset(&alive_addr, 0, sizeof(addr));
             addr.sin_family = AF_INET;
             lin[matches[1].rm_eo] = '\0';
-            if((host = gethostbyname(lin + matches[1].rm_so)) == NULL) {
+            if((host = gethostbyname(lin + matches[1].rm_so)) == NULL || host->h_addr_list[0] == NULL) {
                 logmsg(LOG_ERR, "Unknown back-end host \"%s\"", lin + matches[1].rm_so);
                 exit(1);
             }
@@ -515,7 +592,7 @@ parse_file(char *fname)
             memset(&alive_addr, 0, sizeof(addr));
             alive_addr.sin_family = AF_INET;
             lin[matches[1].rm_eo] = '\0';
-            if((host = gethostbyname(lin + matches[1].rm_so)) == NULL) {
+            if((host = gethostbyname(lin + matches[1].rm_so)) == NULL || host->h_addr_list[0] == NULL) {
                 logmsg(LOG_ERR, "Unknown back-end host \"%s\"", lin + matches[1].rm_so);
                 exit(1);
             }
@@ -577,7 +654,7 @@ parse_file(char *fname)
             /* this matches Cookie: ... as well as Set-cookie: ... */
             snprintf(pat, MAXBUF - 1, "Cookie:.*[ \t]%s=([^;]*)", lin + matches[1].rm_so);
             if(regcomp(&groups[tot_groups]->sess_pat, pat, REG_ICASE | REG_EXTENDED)) {
-                logmsg(LOG_ERR, "Session URL bad pattern \"%s\" - aborted", pat);
+                logmsg(LOG_ERR, "Session Cookie bad pattern \"%s\" - aborted", pat);
                 exit(1);
             }
             if((groups[tot_groups]->sess_to = atoi(lin + matches[2].rm_so)) <= 0)
@@ -647,6 +724,7 @@ parse_file(char *fname)
     regfree(&ListenHTTPS);
     regfree(&HTTPSHeaders);
     regfree(&SSL_CAlist);
+    regfree(&SSL_Verifylist);
 #if HAVE_OPENSSL_ENGINE_H
     regfree(&SSLEngine);
 #endif
@@ -660,6 +738,7 @@ parse_file(char *fname)
     regfree(&ExtendedHTTP);
     regfree(&WebDAV);
     regfree(&NO11SSL);
+    regfree(&LogFacility);
     regfree(&LogLevel);
     regfree(&Alive);
     regfree(&Client);
@@ -671,6 +750,7 @@ parse_file(char *fname)
     regfree(&Err500);
     regfree(&Err501);
     regfree(&Err503);
+    regfree(&Err414);
     regfree(&CheckURL);
     regfree(&CS_SEGMENT);
     regfree(&CS_PARM);
@@ -681,15 +761,23 @@ parse_file(char *fname)
     regfree(&HeadRemove);
     regfree(&HeadRequire);
     regfree(&HeadDeny);
+    regfree(&RewriteRedir);
+    regfree(&NoDaemon);
     return;
 }
 
+extern char *optarg;
+extern int  optind, opterr, optopt;
+
 /*
  * parse the arguments/config file
  */
 void
 config_parse(int argc, char **argv)
 {
+    int c_opt, check_only;
+    char    *conf_name;
+
     /* init values */
     clnt_to = 10;
     server_to = 0;
@@ -697,10 +785,11 @@ config_parse(int argc, char **argv)
     https_headers = 0;
     https_header = NULL;
     ssl_CAlst = NULL;
-    ssl_vdepth = 0;
+    ssl_Verifylst = NULL;
+    ssl_vdepth = 1;
     allow_xtd = 0;
     allow_dav = 0;
-    no_https_11 = 0;
+    no_https_11 = 2;
     alive_to = 30;
     max_req = 0L;
     http = NULL;
@@ -712,25 +801,48 @@ config_parse(int argc, char **argv)
     user = NULL;
     groups = NULL;
     root = NULL;
-    e500 = e501 = e503 = NULL;
+    e500 = e501 = e503 = e414 = NULL;
     CS_segment = CS_parm = CS_qid = CS_qval = CS_frag = NULL;
     head_off = NULL;
     check_URL = 0;
+    rewrite_redir = 1;
+    daemonize = 1;
+    log_facility = LOG_DAEMON;
 
-    if(argc == 1) {
-        /* without arguments - use default configuration file */
-#ifndef F_CONF
-#define F_CONF  "/etc/pound/pound.cfg"
-#endif
-        parse_file(F_CONF);
-    } else if(argc == 3 && !strcmp(argv[1], "-f")) {
-        /* argument is the configuration file */
-        parse_file(argv[2]);
-    } else {
-        logmsg(LOG_ERR, "bad argument(s)");
+    opterr = 0;
+    check_only = 0;
+    conf_name = F_CONF;
+    pid_name = F_PID;
+
+    while((c_opt = getopt(argc, argv, "f:cvp:")) > 0)
+        switch(c_opt) {
+        case 'f':
+            conf_name = optarg;
+            break;
+        case 'p':
+            pid_name = optarg;
+            break;
+        case 'c':
+            check_only = 1;
+            break;
+        case 'v':
+            print_log = 1;
+            break;
+        default:
+            logmsg(LOG_ERR, "bad flag -%c", optopt);
+            exit(1);
+        }
+    if(optind < argc) {
+        logmsg(LOG_ERR, "unknown extra arguments (%s...)", argv[optind]);
         exit(1);
     }
 
+    parse_file(conf_name);
+    if(check_only) {
+        logmsg(LOG_INFO, "Config file %s is OK", conf_name);
+        exit(0);
+    }
+
     if(!http[0] && !https[0]) {
         logmsg(LOG_ERR, "no HTTP and no HTTPS - aborted");
         exit(1);
@@ -750,6 +862,8 @@ config_parse(int argc, char **argv)
         e501 = "This method may not be used.";
     if(e503 == NULL)
         e503 = "The service is not available. Please try again later.";
+    if(e414 == NULL)
+        e414 = "Request URI is too long";
 
 #ifdef  UNSAFE
     if(CS_segment == NULL)
